/* ratioSlider.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "ratioSlider.h"
#include "tableFieldKey.h"
#include "tableField.h"

#include <math.h>

gfloat RatioSlider::
get_value() const {
  if (_active) {
    return _adjustment.get_value();
  } else {
    return 0.0;
  }
}

TableID RatioSlider::
get_table_id() const {
  return _table_id;
}

int RatioSlider::
get_field_id() const {
  return _field_id;
}

RatioSlider::
RatioSlider(RatioSliderGroup *group, TableID table_id, int field_id) :
  _group(group),
  _table_id(table_id),
  _field_id(field_id),
  _label(TableFieldKey(table_id, field_id).get_field()._label),
  _adjustment(1.0, 0.0, 100.0, 1.0, 10.0, 0.0)
{
  _scale.set_adjustment(_adjustment);
  _scale.set_value_pos(GTK_POS_RIGHT);
  _scale.set_digits(1);
  _active = true;
  _check.set_active(true);
  _check.add(_label);
  _check.set_border_width(0);
  _check.set_usize(0, 16);
  _label.show();

  connect_to_method(_adjustment.value_changed, this, &new_value);
  connect_to_method(_check.clicked, this, &toggle_check);
}

void RatioSlider::
new_value() {
  _group->rescale_values(this);
}

void RatioSlider::
toggle_check() {
  _active = _check.get_active();

  if (_active) {
    _scale.show();
    _scale.set_draw_value(true);
  } else {
    _scale.hide();
    _scale.set_draw_value(false);
  }
  new_value();
}

RatioSliderGroup::
RatioSliderGroup() : Gtk_Table(0, 2, false) {
  _rescaling = false;
}

RatioSlider *RatioSliderGroup::
make_slider(TableID table_id, int field_id) {
  int index = _sliders.size();

  resize(index+1, 2);
  RatioSlider *slider = new RatioSlider(this, table_id, field_id);
  attach(slider->_check, 0, 1, index, index+1, GTK_FILL, GTK_SHRINK);
  attach(slider->_scale, 1, 2, index, index+1, GTK_FILL | GTK_EXPAND, 0);

  slider->_check.show();
  slider->_scale.show();

  _sliders.push_back(slider);
  return slider;
}

void RatioSliderGroup::
rescale_values(RatioSlider *except) {
  // If the rescaling flag is true, we've entered this function
  // recursively as the result of fiddling with the adjustment values.
  // That's ok, don't do anything.
  if (_rescaling) {
    return;
  }

  // Now protect ourselves from recursive reentry.
  _rescaling = true;

  // We have some sliders that are fixed, and some that are loose.
  // The total of all of them should be 100.

  // We can't move the fixed sliders, so whatever they total (which is
  // hopefully less than 100) can't be changed.  We'll have to move
  // the loose sliders around the amount we have left to make the
  // total come out to 100.
  
  double net_fixed = 0.0;
  double net_loose = 0.0;
  int num_fixed = 0;
  int num_loose = 0;
  double max_value = 0.0;

  Sliders loose_sliders;

  Sliders::iterator sli;
  for (sli = _sliders.begin(); sli != _sliders.end(); ++sli) {
    if ((*sli)->_active) {
      double value = (*sli)->_adjustment.get_value();
      max_value = max(max_value, value);
      
      if ((*sli) == except) {
	net_fixed += value;
	num_fixed++;
      } else {
	net_loose += value;
	num_loose++;
	loose_sliders.push_back(*sli);
      }
    }
  }

  if (num_loose == 0) {
    // Nothing we can do if there are no loose sliders!

  } else if (net_fixed >= 100.0) {
    // All the available space is taken up by the fixed sliders.  Set
    // everything loose to zero.
    for (sli = loose_sliders.begin(); sli != loose_sliders.end(); ++sli) {
      (*sli)->_adjustment.set_value(0.0);
    }

  } else if (net_loose == 0.0) {
    // There's some available space, but all the loose sliders are
    // zero.  Scale them all up into the available space.
    double avg_amount = (100.0 - net_fixed) / num_loose;
    for (sli = loose_sliders.begin(); sli != loose_sliders.end(); ++sli) {
      (*sli)->_adjustment.set_value(avg_amount);
      max_value = max(max_value, avg_amount);
    }

  } else {
    // There's some available space; scale into it.
    double factor = (100.0 - net_fixed) / net_loose;
    for (sli = loose_sliders.begin(); sli != loose_sliders.end(); ++sli) {
      Gtk_Adjustment &adj = (*sli)->_adjustment;
      double value = adj.get_value() * factor;
      max_value = max(max_value, value);
      adj.set_value(value);
    }
  }

  // Finally, we'd like to avoid having all the sliders mushed up
  // against the left edge of the available range--it's hard to edit
  // in there.  We'll scale the available range to keep the maximum
  // slider somewhere in the middle or on the right.

  // This does distort somewhat our perception of the relative
  // importance of the various sliders.  But I think it makes it
  // easier to work with.

  double top_value = 100.0;
  if (max_value < 98.0) {
    top_value = 100.0 * pow(max_value / 100.0, 0.6);
  }

  for (sli = _sliders.begin(); sli != _sliders.end(); ++sli) {
    if ((*sli)->_active) {
      Gtk_Adjustment &adj = (*sli)->_adjustment;
      adj.gtkobj()->upper = top_value;
      adj.set_value(adj.get_value());
    }
  }

  _rescaling = false;
}

