/* circularBuffer.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "circularBuffer.h"

#include <assert.h>
#include <ctype.h>

CircularBuffer::
CircularBuffer(int max_size) {
  // New data is added to the buffer beginning at _in.
  // Data is removed from the buffer beginning at _out.

  // The buffer is empty when _out == _in.
  // The buffer is full when (_out == _in - 1) || (_out == 0 && _in == _top)

  // Note that _buffer[_top] is the last valid character in the buffer.
  _top = max_size;
  _buffer = new char[_top + 1];

  _in = _out = 0;
}

CircularBuffer::
~CircularBuffer() {
  delete[] _buffer;
}

int CircularBuffer::
get_max_size() const {
  return _top;
}

int CircularBuffer::
get_current_size() const {
  if (_in >= _out) {
    return _in - _out;
  } else {
    return ((_top+1) - _out) + _in;
  }
}

bool CircularBuffer::
is_empty() const {
  return (get_current_size() == 0);
}

bool CircularBuffer::
is_full() const {
  return (get_current_size() == get_max_size());
}

double CircularBuffer::
get_percent_full() const {
  return 100.0 * (double)get_current_size() / (double)get_max_size();
}

int CircularBuffer::
get_read_size() const {
  if (_in >= _out) {
    return _in - _out;
  } else {
    return (_top+1) - _out;
  }
}

const char *CircularBuffer::
read() const {
  return _buffer + _out;
}

void CircularBuffer::
extract_characters(int num_characters) {
  assert(num_characters >= 0 && num_characters <= get_read_size());
  _out = (_out + num_characters) % (_top+1);
}


int CircularBuffer::
get_write_size() const {
  // We must never allow the buffer to completely fill, because that
  // will make _in == _out and the buffer will seem to be empty.

  if (_out == 0) {
    return _top - _in;
  } else if (_in >= _out) {
    return (_top+1) - _in;
  } else {
    return _out - _in - 1;
  }
}

char *CircularBuffer::
write() {
  return _buffer + _in;
}

void CircularBuffer::
insert_characters(int num_characters) {
  assert(num_characters >= 0 && num_characters <= get_write_size());
  _in = (_in + num_characters) % (_top+1);
}

inline char
clean_char(char c) {
  if (isprint(c)) {
    return c;
  } else {
    return '*';
  }
}
  

void CircularBuffer::
describe(ostream &out) const {
  out << "CircularBuffer, max_size = " << get_max_size()
      << " current_size = " << get_current_size() << "\n"
      << "  read_size = " << get_read_size() 
      << " write_size = " << get_write_size() << "\n"
      << "  _in = " << _in << " _out = " << _out << ":\n";
  if (_in >= _out) {
    int i;
    for (i = 0; i < _out; i++) {
      out << '-';
    }
    for (i = _out; i < _in; i++) {
      out << clean_char(_buffer[i]);
    }
    for (i = _in; i <= _top; i++) {
      out << '-';
    }
  } else {
    int i;
    for (i = 0; i < _in; i++) {
      out << clean_char(_buffer[i]);
    }
    for (i = _in; i < _out; i++) {
      out << '-';
    }
    for (i = _out; i <= _top; i++) {
      out << clean_char(_buffer[i]);
    }
  }    
  out << "\n";
}
